/****************
* Glob the Blob *
*****************
* Camera        *
****************/

function CameraSlide(from_x, from_y, to_x, to_y, relative, slide_back, freeze, wait_pause, duration)
{
  /*******************************************************************
    EXTERNAL/CALLABLE FUNCTION
  ********************************************************************
    Slides the camera from initial map coordinates to the final
    coordinates over a gradual period of "duration" milliseconds.
  *******************************************************************/
  
  var cam_person = "";
  var input_person = "";
  
  var old_x = 0;
  var old_y = 0;
  var start_x = 0;
  var start_y = 0;
  var finish_x = 0;
  var finish_y = 0;
  if(!relative)
  {
	to_x=to_x*16+160;
	to_y=to_y*16+120;
  }
  else
  {
	to_x*=16;
	to_y*=16;
  }
  var start_time = 0;
  var current_time = 0;
  var last_time = 0;
  var updates = 0;
  var frames = 0;
  var frame_rate = GetMapEngineFrameRate();
  
  // Take the camera away from the default focus
  if (IsCameraAttached())
  {
    cam_person = GetCameraPerson();
    DetachCamera();
  }
  
  // Take input too if needed
  if (freeze && IsInputAttached())
  {
    input_person = GetInputPerson();
    DetachInput();
  }
  
  // Prepare coordinates
  old_x = GetCameraX();
  old_y = GetCameraY();
  
  if (relative)
  {
    start_x = old_x + from_x;
    start_y = old_y + from_y;
    finish_x = old_x + to_x;
    finish_y = old_y + to_y;
  }
  else
  {
    start_x = from_x;
    start_y = from_y;
    finish_x = to_x;
    finish_y = to_y;
  }
  
  // Move the camera
  start_time = GetTime();
  current_time = start_time;
  last_time = start_time;
  
  while (current_time < start_time + duration)
  {
    // Update current time
    current_time = GetTime();
    
    // Use time to calculate where the camera should be
    SetCameraX(Round(start_x + (finish_x - start_x) * (current_time - start_time) / duration));
    SetCameraY(Round(start_y + (finish_y - start_y) * (current_time - start_time) / duration));
    
    // Draw stuff at usual framerate
    updates += (GetTime() - last_time) * frame_rate / 1000;
    for(; frames < updates; frames++)
    {
      UpdateMapEngine();
    }
    last_time = GetTime();
    
    RenderMap();
    FlipScreen();
  }
  
  // Wait for the specified time
  if (wait_pause < 0)
  {
    // Wait for a key
    while (IsAnyKeyPressed())
    {
      // Draw stuff at usual framerate
      updates += (GetTime() - last_time) * frame_rate / 1000;
      for(; frames < updates; frames++)
      {
        UpdateMapEngine();
      }
      last_time = GetTime();
      
      RenderMap();
      FlipScreen();
    }
    while (!IsAnyKeyPressed())
    {
      // Draw stuff at usual framerate
      updates += (GetTime() - last_time) * frame_rate / 1000;
      for(; frames < updates; frames++)
      {
        UpdateMapEngine();
      }
      last_time = GetTime();
      
      RenderMap();
      FlipScreen();
    }
  }
  else
  {
    // Wait for the specified time
    start_time = GetTime();
    do
    {
      // Draw stuff at usual framerate
      updates += (GetTime() - last_time) * frame_rate / 1000;
      for(; frames < updates; frames++)
      {
        UpdateMapEngine();
      }
      last_time = GetTime();
      
      current_time = GetTime();
      RenderMap();
      FlipScreen();
    } while (current_time < start_time + wait_pause);
  }
  
  // Slide it back to the old coordinates if neccessary
  if(slide_back)
  {
	  start_time = GetTime();
	  current_time = start_time;
	  last_time = start_time;
	  
	  while (current_time < start_time + duration)
	  {
	    // Update current time
	    current_time = GetTime();
	    
	    // Use time to calculate where the camera should be
	    SetCameraX(Round(finish_x + (start_x - finish_x) * (current_time - start_time) / duration));
	    SetCameraY(Round(finish_y + (start_y - finish_y) * (current_time - start_time) / duration));
	    
	    // Draw stuff at usual framerate
	    updates += (GetTime() - last_time) * frame_rate / 1000;
	    for(; frames < updates; frames++)
	    {
		UpdateMapEngine();
	    }
	    last_time = GetTime();
	    
	    RenderMap();
	    FlipScreen();
	  }
	  
	  // Return the camera to the owner
	  if (cam_person != "")
	  {
	    AttachCamera(cam_person);
	  }
	  
	  // Return input
	  if (input_person != "")
	  {
	    AttachInput(input_person);
	  }
  }
  return 0;
}

function CameraZoomTo(x, y, zoom_percent, travel_time, wait_pause)
{
  /*******************************************************************
    EXTERNAL/CALLABLE FUNCTION
  ********************************************************************
    Slides the camera to the specified position over a gradual
    period of "travel_time" milliseconds. The camera immediately
    zooms in to the target in a split second to the specified zoom,
    and either waits for "wait_pause" milliseconds, or for a key
    press if "wait_pause" is under 0.
  *******************************************************************/
  
  return 0;
}
