//----------------------------
//REGULAR MOVEMENT
//----------------------------

//___________________________________
//                                   \
// MOVEMENT CORE. Glues all together.
//___________________________________/

//___________________________________
//                                   \
  RequireScript("lithonite.js");
//___________________________________/


//----------------------------------------------------------------------------//

/*
	This is the only way to declare Lithonite. 
	(you could also declare it like: var L= new LithoniteEngine("L");
	as long as those 2 names are the same).
	A second parameter is optional: If you will never change the main character:
	var L= new LithoniteEngine("L", 'bob');
*/
var Lithonite= new LithoniteEngine("Lithonite");

/*
	The default pixels lookup value for anti-obstruction is 8, make it bigger so
	the effect is more obvious.
*/
Lithonite.LMaxLookupX=10;
Lithonite.LMaxLookupY=13;

/*
	short-write slope(), just like: sin=Math.sin;
	Only this doesnt work: slope=Lithonite.slope;
	Because SLOPES contains references to "this.", which becomes non-defined outside
	the Lithonite Object.
*/
function slope(c){Lithonite.slope(c)};

/*
	Add you custom-made slope type (thing to run inside a Zone). See slope() in lithonite.js
	This example swaps the cursor keys (left & up, right & down).
	Its defined in the Zone at the upper-right corner of this demo.
	Note that zones have a value of 8 wait cycles before triggering, so it possible that you can enter a zone up to 8 
	pixels before it triggers. The lower the number, the slower the game though. You can make the zone a little bit 
	bigger, but that is not always possible
*/
Lithonite.SLOPES['myslope'] = function(){
	if(!this.moving)return; //We dont really need this line, its just to make the function faster.
	this.active=false;  //Lithonite deObstruct() works with the real move_x/move_y, so its in our way.
	var newX = -this.move_x+this.move_y; //Make up a new direction X
	var newY = -this.move_y+this.move_x; //Make up a new direction Y
	SetPersonDirection(this.GIP,this.getMoveDir(this.hist_y,this.hist_x)); //Face this new direction

	//Also try this, it somewhat replaces this whole block of repos() and deObstruct(): 
	// this.shove(GetPersonSpeedX(this.GIP)*newX,GetPersonSpeedY(this.GIP)*newY);
	if(! this.repos(GetPersonSpeedX(this.GIP)*newX,GetPersonSpeedY(this.GIP)*newY) ) { //Try repositioning our person
		this.active=true; //We Couldnt reposition it, so lets use deObstruct() to help us
		this.move_x = this.move_y; //Set fake move_x used by deObstruct() 
		this.move_y = this.move_x; //Set fake move_y used by deObstruct() 
		this.deObstruct(); // deObstruct us. 
		this.active=false; //Disable deObstruct() again.
	}
};


var Do_StatBar=false;	//Debug Status displayed
var showhelp=false;	//Help displayed

/*
	The keys used in this game, wrapped together in an object.
*/
var Keys = {
	talk:	KEY_CTRL,
	run:	KEY_SHIFT,
	pause:	KEY_P,
	inventory: KEY_ALT,
	menu:	KEY_ESCAPE,
	qsave:	KEY_F2,
	qload:	KEY_F3,
	help:	KEY_F1
}


//----------------------------------------------------------------------------//
/*
	In order to activate all extra things each drawn frame, we need to hook
	our functions.

*/
SetUpdateScript("Movement_Update()");	// calls 'Movement_Update()' before all map layers are rendered
SetRenderScript("Render_Update()");	// calls 'Render_Update()'   after  all map layers are rendered

//----------------------------------------------------------------------------//


function Movement_Update() 
{
 OnKEYDown();
 if (Lithonite.justStopped()) OnKEYUp();
 //The following is because talk is only checked in directions north/south/west/east and not in face_north/etc.
 if(IsKeyPressed(GetTalkActivationKey())&&Lithonite.moving==0){
		QueuePersonCommand(Lithonite.GIP, Lithonite.getCommand(0,Lithonite.hist_x,Lithonite.hist_y),true);
		Lithonite.moving=1;
  }
}




//----- KEY UP AND KEY DOWN -----//
function OnKEYDown()
{

 //#Read Inputbuffer for NON-retriggering events.(for example: toggle's)
 while (AreKeysLeft()) 
 {
   switch (GetKey()) 
  {
   case KEY_ESCAPE: Exit(); break;
   case KEY_F1:	showhelp=!showhelp; break;
   case KEY_F5:	RestartGame();break;
   case KEY_S:	Do_StatBar=!Do_StatBar; break;
   case KEY_I:	IgnorePersonObstructions(MainChar, true); break;
   case KEY_L:	Lithonite.active=!Lithonite.active; break; //Toggle Lithonite on/off
   case KEY_W:  Lithonite.SwapDirections('robe_'); break; // toggle robe on/off
   case KEY_R:  Lithonite.LoadDirections(); break; //restore original directions
   case KEY_E:  Lithonite.SaveDirections(); break; //save current directions as originals.
   case KEY_K: Lithonite.repos(0,0); break; //Recenter: Truncate your position so its not a Float.
  }//end_switch
 }//end_while 

 //#Read input buffer for Retriggering events (it will loop as long as you hold the key pressed).
 if (IsKeyPressed(KEY_D)) Do_StatBar=!Do_StatBar;  //The wrong place to put this, Press S / D to see the difference.

 // Calling this each time Movement_Update() is called is redundant, 
 // but its very safe. All the actions like checkDashing() and 
 // calcVectors() are checked against this InputPerson.
 // So if you never will swap your input person, you can call this 
 // once. In game(), for example, with parameters, like this:
 // Lithonite.setInputPerson(MainChar);
 Lithonite.setInputPerson();

 //#DashAlgorithm
 Lithonite.checkDashing(Keys.run);

 Lithonite.calcVectors(); //Calculate the Lithonite vectors.
 Lithonite.doSlope();     //Use to add movement effects in a zone.

}//End_Function:OnKEYDown


function OnKEYUp(){
  Lithonite.standStill(); //Set a face_<> direction and unsets justStopped(), justStopped() is now false again.
}

//----- STATUS BAR + Weather effects + Overworld effect-----//
function Render_Update()
{
 //Here go all UpdateAndDraw() functions. This demo has none.
 Lithonite.deObstruct(); //Help player walk through corners. (returns true if it had to help)

 var debugfont=GetSystemFont();

 if (showhelp)
 {
  var text=
	"Press S to toggle status bar\n"+
	"Press D to trigger status bar\n"+
	"Press L to toggle lithonite on/off\n"+
	"Press K to recenter Celes\n"+
	"Press W to swap outfit\n"+
	"Press E to set original outfit\n"+
	"Press R to retore original outfit\n"+
	"Press F5 to restart the demo\n"+
	" Press ESC to exit\n";
  	debugfont.drawTextBox(10, 10, 230, 310, 0, text);
	debugfont.drawText(10,230," Press F1 to hide the help");
	debugfont.drawText(11,230," Press F1 to hide the help"); //BOLD!
 }
 else debugfont.drawText(10,230," Press F1 to show the help");

 if (Do_StatBar)
 {
   var debugwindow=GetSystemWindowStyle();
   debugwindow.drawWindow(220,16,80,40);
   DrawPersonBases();
   debugfont.drawText(10,10,sDebug);
   debugfont.drawText(10,20,nDebug);
   var PERSON=IsInputAttached()?GetInputPerson():Lithonite.GIP;

   Rectangle(5, 170, GetScreenWidth()-5*2, 62, CreateColor(0,0,0,200) );
   debugfont.drawText(260,40,"c?="+IsInputAttached());

   //Position Relative to MAP
	debugfont.drawText(205,10, " X="+GetPersonXFloat(PERSON));
	debugfont.drawText(205,20, " Y="+GetPersonYFloat(PERSON));
	
   //Position Relative to SCREEN
	debugfont.drawText(265,10, " H"+MapToScreenX(0,GetPersonX(PERSON)));
	debugfont.drawText(265,20, " V"+MapToScreenY(0,GetPersonY(PERSON)));

   //Position Relative to (-0,-0) of MAP
	var X0 = (GetScreenWidth>>1)-MapToScreenX(0,GetPersonX(PERSON))+GetPersonX(PERSON);
	var Y0 = (GetScreenHeight>>1)-MapToScreenY(0,GetPersonY(PERSON))+GetPersonY(PERSON);
	debugfont.drawText(205,35, " X'="+ X0);
	debugfont.drawText(205,45, " Y'="+ Y0);

  	debugfont.drawText(200,220," Layer="+ GetPersonLayer(PERSON) );
  
	debugfont.drawText(5,200,
		"Lithonite Vectors: X="+Lithonite.move_x+
		" Y="+Lithonite.move_y+
		" L?="+Lithonite.active + " Idle="+Lithonite.idle
	);

	debugfont.drawText(5,210,
	"P="+Lithonite.pushing+" HX="+Lithonite.hist_x+" HY="+Lithonite.hist_y+
		" SpeedX="+Math.round(GetPersonSpeedX(PERSON)*1000)/1000+
		" SpeedY="+Math.round(GetPersonSpeedY(PERSON)*1000)/1000
	);

	debugfont.drawText(5,180,
		" CommandQueueEmpty="+IsCommandQueueEmpty(Lithonite.GIP)+
		" moving="+Lithonite.moving+
		" zone="+Lithonite.zone +" "
	);

	debugfont.drawText(5,190,
		" IX="+Lithonite.i_x + " IY="+Lithonite.i_y 
	);


	if(IsInputAttached() ) 
	{
		var person=GetInputPerson();
		sprite=GetInputPerson();
		var $GPX = GetPersonX(person);
   		var $GPY = GetPersonY(person);
   		var $GPXX= $GPX+Lithonite.hist_x;
   		var $GPYY= $GPY+Lithonite.hist_y;
		if(IsPersonObstructed(person,$GPXX,$GPYY))
  		{
			sprite+=" "+GetObstructingPerson(person, $GPXX, $GPYY);
		}
	}
	else sprite="";
	debugfont.drawText(0,170," "+sprite);
 }//#EndIf Statbar

}


//----------------------------------------------------------------------------//

// *** Debug functions (borrowed from esthetica.js) *** //

var sDebug='';
var nDebug='';

function DrawPersonBases() { //See also DrawSpriteBases(), which edits the images and is not realtime.
  var p = GetPersonList();
  var i=p.length-1;
  do {DrawPersonBase(p[i]);}while(i--);
}

function DrawPersonBase(person) {

  if(!DoesPersonExist(person)) return false;

  var layer = GetPersonLayer(person);
  var base=GetPersonBase(person);
  var dx=(base.x2-base.x1+1)
  var dy=(base.y2-base.y1+1)
  var a1=MapToScreenX(layer,GetPersonX(person)-(dx>>1)+1)
  var b1=MapToScreenY(layer,GetPersonY(person)-(dy>>1))
  var a2=MapToScreenX(layer,GetPersonX(person)+(dx>>1))
  var b2=MapToScreenY(layer,GetPersonY(person)+(dy>>1))
    
  var lcol = CreateColor(255, 0, 255);

  //lcol2= CreateColor(10, 0, 0,100);
  //Rectangle(a1, b1,dx, dy, lcol2);

  Line(a1,b1, a2, b1, lcol);
  Line(a1,b1, a1, b2, lcol);
  Line(a1,b2, a2, b2, lcol);
  Line(a2,b1, a2, b2, lcol);
}	

function DoesPersonExist(person)
{
  var p = GetPersonList();
  var i=p.length-1;
  do {if(p[i] == person) return true;}while(i--);
  return false;
}

